// *****************************************************************************
//
//	File		:  stiquitoMSP430.c
//
//	Author		:	Andrew McClain
//
//	Description:	Includes main and supporting functions for Stiquito
//					basic operation
//
//	Features/Information:
//			-Texas Instruments MSP430F1122 MicroProcessor
//			-Supports dynamic switching between 1-dof and 2-dof modes
//				based on jumper setting
//			-Supports dynamic speed change based on on-board potentiometer
//				(through 10-bit A/D converter).  Complete leg cycle ranges
//				from 3-11secs
//
//	Terminology	:	Cycle - One complete operation of all legs.  Once each leg
//						has been pulled back and (in 2-dof, pulled forward)
//						returned to inital state, one cycle is complete.
//					Stage - One 16th of a cycle.
//					Step - One 16th of a Stage.
//
//	Revisions
//------------------------------------------------------------------------------
//
//	Version 1.0
//	Developers	:	Andrew McClain
//	Date		:	10 Nov 2004
//	Purpose	:	Create initial code for Stiquito that satisfies requirements
//					stated in Stiquito book.
//
//------------------------------------------------------------------------------
//
//	Version 1.1
//	Developers	:	Andrew McClain
//	Date		:	22 Nov 2004
//	Purpose	:	Tidy code and comment for readability/future expansion
//
//------------------------------------------------------------------------------
//
//  Version 2.0
//
//  Developers	:	Andrew McClain
//	Date		:	31 May 2005
//	Purpose		:	Redesign output to reduce code size and allow adjustment of
//					duty cycle for outputs.
//
//	New Terminology:	Cycle - One complete operation of all legs
//						Stage - One 16th of a cycle
//						Step - Number of times the output pattern is repeated
//								per stage.  In version 2.0 , there are 3
//								steps per stage.
//						Substep - Number of increments in one pattern.  In
//								version 2.0, patterns contain 5 substeps
//								for 20% duty cycle precision
//
//	Further Design:
//			- Only 1/2 (4 bits) of the P1 port are written to.  Investigate
//				possibility of compressing output pattern array to use only
//				half of storage space.  (Currently stores pattern in one full
//				byte per pattern)
//			- Add LED out.  Possibly a toggle function that was present in
//				previous 1.x versions.
//			- Calculate new speed equation.  In speed set function, replace the
//				line "d= d*78+859" with new slope and offset values calculated
//				from 2 data points from oscilloscope.  (In this case, 78 is the
//				slope, 859 is offset) The statement "d= m/32 + 1;" does not need
//				to be altered; it creates a value from 1-32 based on the
//				0-1023 A/D register value.  Also reset default value.  This
//				value is triggered in the event non-valid results are obtained
//				from the A/D.
//			- Set up degree of freedom functionality.  Would require either
//				a)  a second pattern table
//				b)  masking of original pattern table
//				c)  change P1DIR to mask secondary tripod outputs
//				Both b and c assume that the output of the primary tripods does
//				not change from 1 to 2 degree of freedom movement.  This is
//				not the case in original design.  Originally, tripods were
//				active longer to prevent reverse locomotion.
//			- Design bit patterns to reduce multiple tripods being active
//				simultaneously.  For example, for stages where a tripod is only
//				active for 1 out of 5 substeps, some tripods can be active for
//				the first substep, while others are active for the 2nd, etc.
//				Depending on duty cycle desired, it may not be possible to fully
// 				avoid collision.
//
// *****************************************************************************

#include <msp430x11x2.h>

// *************************
// *  Function Prototypes  *
// *************************
//
// See function descriptions at function code headers

void set_new_speed (void);
void set_speed (void);

// **********************
// *  Global Variables  *
// **********************

int degree =2;		// Determines degree-of-freedom.  Start with 2 dof, but
					// jumper is checked before legs are moved.

int stage = 0;  	// Tracks the stage (0-15) of cycle.

int step = 0;		// Tracks the steps (0-3) within the stage.

int substep = 0;	// Tracks the substep (0-5) within the output pattern

int speed_reg; 		// Value used to continuously increment speed register,
					// set by set_new_speed().

int current_speed = -1; // fastest speed setting (1-32)

char pattern[16][5]  	// output byte patterns for each step of each stage
						// format is pattern[stage#][substep#]
	={	{ 0x09, 0x08, 0x00, 0x00, 0x00 },
		{ 0x09, 0x00, 0x00, 0x00, 0x00 },
		{ 0x08, 0x00, 0x00, 0x00, 0x00 },
		{ 0x08, 0x00, 0x00, 0x00, 0x00 },
		{ 0x0A, 0x02, 0x00, 0x00, 0x00 },
		{ 0x0A, 0x00, 0x00, 0x00, 0x00 },
		{ 0x02, 0x00, 0x00, 0x00, 0x00 },
		{ 0x02, 0x00, 0x00, 0x00, 0x00 },
		{ 0x06, 0x04, 0x00, 0x00, 0x00 },
		{ 0x06, 0x00, 0x00, 0x00, 0x00 },
		{ 0x04, 0x00, 0x00, 0x00, 0x00 },
		{ 0x04, 0x00, 0x00, 0x00, 0x00 },
		{ 0x05, 0x01, 0x00, 0x00, 0x00 },
		{ 0x05, 0x00, 0x00, 0x00, 0x00 },
		{ 0x01, 0x00, 0x00, 0x00, 0x00 },
		{ 0x01, 0x00, 0x00, 0x00, 0x00 },	};


// **********
// *  Main  *
// **********

void main (void) {

	BCSCTL1 &= ~(RSEL0 + RSEL1 + RSEL2);	//Sets RSELx bits to 000, slowest speed
	DCOCTL  &= ~(DCO0 + DCO1 + DCO2);	// Sets DCOx bits to 000, slowest speed

	WDTCTL = WDT_MDLY_32;	// WDT 1s interval timer.  This statment
							// is required even though WDT is not implemented

	CCTL0 = CCIE;	// CCR0 interrupt enabled
	TACTL = TASSEL_2 + MC_2;	// SMCLK, contmode.  See TI sample code for description

	// P1 is set as follows:
	// P1.1:  Tripod A 2-dof - output
	// P1.0:  Tripod B 2-dof - output
	// P1.2:  Tripod B - output
	// P1.3:  Tripod A - output
	// P1.4 thru P1.7 : JTAG -Input

	P1DIR = 0x0F;	// Init P1.x
	P1OUT = 0x00;	// Turn off legs and other outputs

	// P2 is set as follows:
	// P2.0: ADC input from POT to adjust speed
	// P2.2 and P2.0 : routed to Protype Area
	// P2.3:  Tripod B LED - output
	// P2.4:  Tripod A LED - output
	// P2.5:  2-dof jumper - input

	P2DIR = 0x1E;	// Init P2.x
	P2OUT = 0x00;	//turn off legs and other outputs
	P2OUT |= 0x18;	// turn off leds (inverse logic)

	set_new_speed();	// Get initial pot and dof jumper settings and set operation
						// accordingly

	CCR0 = speed_reg;	// Set initial timer count register

	__enable_interrupt();	// Enable global interrupts (Start Timer)

	_BIS_SR(LPM0_bits + GIE);	// Enter LPM0 w/ interrupt (Low power standby)

}


// ***************
// *  Functions  *
// ***************


// *****************************************************************************
//
//	Name		:	watchdog_timer
//
//	Description	:	WDT interrupt service routine.  Not currently implemented
//
// *****************************************************************************

#pragma vector=WDT_VECTOR
__interrupt void watchdog_timer (void) {

}	// end void watchdog_timer(void)


// *****************************************************************************
//
//	Name        :	Timer_A3
//
//	Description :	Timer A0 interrupt service routine
//					This function occurs whenever CCR0 register reaches 0.  (This
//					register is then incremented inside the isr)  Executes
//					movement based on global dof flag.  Increments step and stage
//					(if necessary).
//
// ****************************************************************************

#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A3 (void)
{
	CCR0 += speed_reg;	// Reincrement timer

	P1OUT = pattern[stage][substep];

	substep++;	// Increment step number

	// if substep limit has been reached (end of bit pattern),
	//    reset substep count and repeat pattern
	if (substep == 5) {
		substep = 0;
		step++;
	}

	// if step limit has been reached (end of stage),
	//    reset step count and increment stage.
	if (step == 4) {
		step = 0;
		stage++;
		set_new_speed();	// check speed and dof at end/beginning of each stage
		if (stage == 16) {	// if cycle is complete, reset stage #
			stage = 0;
		}

	}	// end of step/stage increment

} // end of timer_isr



// *****************************************************************************
//
//	Name        :	set_speed
//
//	Description :	This function sets up and activates ADC.  Result is obtained
//					and calculation is done to compute speed.  The speed_reg
//					variable is then set to the new value.
//
// ****************************************************************************

void set_speed() {

	int m,d = 0;

	ADC10CTL0 = ADC10ON;	// ADC10ON
	ADC10AE |= 0x01;	// P2.0 ADC option select
	ADC10CTL0 |= ENC + ADC10SC;	// Sampling open

	while ((ADC10CTL1 & ADC10BUSY) == 1);  // Wait for ADC to complete capture

	m = ADC10MEM & 0x03FF;	// Get 10 bit ADC value from result register

	if (m<1024){	// Makes sure value is in valid range (this should always be true)

       d= m/32 + 1;	// Make speed index from 1 to 32
       d= d*78+859;	// Calculate register value
       }
      else d = 937;	// If ADC value is not in range, default to fastest speed.

      speed_reg = d;	// Set global variable.  Will be used next timer isr to
      					// increment timer register

}	// end of set_speed

// *****************************************************************************
//
//	Name        :	set_new_speed
//
//	Description :	This function calls set_speed to get new speed from ADC and
//					changes degree of freedom based on jumper.  Because the 2-dof
//					legs are offset in steps, a check is required for certain
//					stages to prevent a 2-dof leg from being stuck on (inverted)
//
// ****************************************************************************

void set_new_speed() {

	set_speed();	// Set speed based on potentiometer

	if (P2IN & 0x20)	// Check if Port 2.5 is jumpered
    {
		degree = 1;	// Set new dof variable

	} else {

		degree = 2;         // for two degree P2.5 low

	}	// end of pin check if statment

}	// end of set_new_speed


